==> jils-0.2.0.tgz: package/lib/core.js <==
var env = require('./env');

module.exports = {
	numberp: numberp,
	stringp: stringp,
	undefinedp: undefinedp,
	nullp: undefinedp,
	arrayp: arrayp,
	functionp: functionp,
	hashp: hashp,
	slice: slice,
	plus: plus,
	minus: minus,
	mult: mult,
	div: div,
	define: define,
	get: get,
	getf: getf,
	assoc: assoc,
	eqp: eqp,
	equalp: equalp,
	arrayeq: arrayeq,
	hasheq: hasheq,
	eq: eq,
	gt: gt,
	gte: gte,
	lt: lt,
	lte: lte,
	nth: nth,
	length: length,
	cons: cons,
	car: car,
	cdr: cdr,
	append: append,
	mapf: mapf,
	reducef: reducef
};

function numberp(num){
	return typeof num == "number";
}

function stringp(str){
	return typeof str == "string";
}

function undefinedp(und){
	return typeof und == "undefined";
}

function arrayp(arr){
	return arr instanceof Array;
}

function functionp(fun){
	return fun instanceof Function;
}

function hashp(has){
	return has instanceof Object &&
		!(arrayp(has)) &&
			!(functionp(has));
}

function slice(arr){
	return Array.prototype.slice.call(arr);
}

function plus(){
	return slice(arguments).reduce(function(a,b){
		return a + b;
	});
}

function minus(){
	var args = slice(arguments);
	if (args.length === 1) args.unshift(0);
	return args.reduce(function(a,b){
		return a - b;
	});
}

function mult(){
	return slice(arguments).reduce(function(a,b){
		return a * b;
	});
}

function div(){
	return slice(arguments).reduce(function(a,b){
		return a / b;
	});
}

function define(env, key, value){
	return (env.values[key] = value);
}

function get(hash, key){
	if (hashp(hash))
		if (hash.hasOwnProperty(key))
			return hash[key];
}

function getf(list, key, deflt){
	var hash = {};
	list.forEach(function(kov,i){
		if (i % 2) hash[lastk] = kov;
		else lastk = kov.replace(/^:/,'');
	});
	return hash[key.replace(/^:/,'')] || deflt;
}

function assoc(hash, ey, value){
	var nhash = {};
	for (var key in hash)
		if (hash.hasOwnProperty(key))
			nhash[key] = hash[key];
	nhash[key] = value;
	return nhash;
}

function eqp(a, b){
	return a === b;
}

function equalp(a, b){
	if (typeof(a) != typeof(b))
		return false;
	if (arrayp(a))
		return arrayp(b) && 
			arrayeq(a, b);
	if (hashp(a))
		return hashp(b) &&
			hasheq(a, b);
	return a == b;
}

function arrayeq(a, b){
	if (a.length != b.length)
		return false;
	for (var i=0,l=a.length;i<l;i++)
		if (a[i] != b[i])
			return false;
	return true;
}

function hasheq(a, b){
	return arrayeq(env.keys(a),env.keys(b)) &&
		arrayeq(values(a),values(b));
}

function eq(a, b){
	return a == b;
}

function gt(a, b){
	return a > b;
}

function gte(a, b){
	return a >= b;
}

function lt(a, b){
	return a < b;
}

function lte(a, b){
	return a <= b;
}

function nth(arr, n){
	if (arrayp(arr))
		return arr[n];
}

function length(arr){
	if (arrayp(arr))
		return arr.length;
	if (hashp(arr))
		return len(env.keys(arr));
}

function cons(a, as){
	return [a].concat(as);
}

function car(x){
	if (x.length) return x[0];
}

function cdr(x){
	if (x.length > 1) return x.slice(1);
}

function append(a, as){
	return a.concat(as);
}

function mapf(fn, arr){
	return arr.map(fn);
}

function reducef(fn, arr, init){
	return arr.reduce(function(a, b){
		return fn(a, b);			
	},init);
}

==> jils-0.3.0.tgz: package/lib/core.js <==
var env = require('./env');

module.exports = {
	numberp: numberp,
	stringp: stringp,
	undefinedp: undefinedp,
	nullp: undefinedp,
	arrayp: arrayp,
	functionp: functionp,
	hashp: hashp,
	slice: slice,
	plus: plus,
	minus: minus,
	mult: mult,
	div: div,
	define: define,
	get: get,
	getf: getf,
	assoc: assoc,
	eqp: eqp,
	equalp: equalp,
	arrayeq: arrayeq,
	hasheq: hasheq,
	eq: eq,
	gt: gt,
	gte: gte,
	lt: lt,
	lte: lte,
	nth: nth,
	length: length,
	cons: cons,
	car: car,
	cdr: cdr,
	append: append,
	mapf: mapf,
	reducef: reducef
};

function numberp(num){
	return typeof num == "number";
}

function stringp(str){
	return typeof str == "string";
}

function undefinedp(und){
	return typeof und == "undefined";
}

function arrayp(arr){
	return arr instanceof Array;
}

function functionp(fun){
	return fun instanceof Function;
}

function hashp(has){
	return has instanceof Object &&
		!(arrayp(has)) &&
			!(functionp(has));
}

function slice(arr){
	return Array.prototype.slice.call(arr);
}

function plus(){
	return slice(arguments).reduce(function(a,b){
		return a + b;
	});
}

function minus(){
	var args = slice(arguments);
	if (args.length === 1) args.unshift(0);
	return args.reduce(function(a,b){
		return a - b;
	});
}

function mult(){
	return slice(arguments).reduce(function(a,b){
		return a * b;
	});
}

function div(){
	return slice(arguments).reduce(function(a,b){
		return a / b;
	});
}

function define(env, key, value){
	return (env.values[key] = value);
}

function get(hash, key, deflt){
	if (hashp(hash))
		if (hash.hasOwnProperty(key))
			return hash[key];
		else return deflt;
}

function getf(list, key, deflt){
	var hash = {};
	list.forEach(function(kov,i){
		if (i % 2) hash[lastk] = kov;
		else lastk = kov.replace(/^:/,'');
	});
	return hash[key.replace(/^:/,'')] || deflt;
}

function assoc(hash, ey, value){
	var nhash = {};
	for (var key in hash)
		if (hash.hasOwnProperty(key))
			nhash[key] = hash[key];
	nhash[key] = value;
	return nhash;
}

function eqp(a, b){
	return a === b;
}

function equalp(a, b){
	if (typeof(a) != typeof(b))
		return false;
	if (arrayp(a))
		return arrayp(b) && 
			arrayeq(a, b);
	if (hashp(a))
		return hashp(b) &&
			hasheq(a, b);
	return a == b;
}

function arrayeq(a, b){
	if (a.length != b.length)
		return false;
	for (var i=0,l=a.length;i<l;i++)
		if (a[i] != b[i])
			return false;
	return true;
}

function hasheq(a, b){
	return arrayeq(env.keys(a),env.keys(b)) &&
		arrayeq(values(a),values(b));
}

function eq(a, b){
	return a == b;
}

function gt(a, b){
	return a > b;
}

function gte(a, b){
	return a >= b;
}

function lt(a, b){
	return a < b;
}

function lte(a, b){
	return a <= b;
}

function nth(arr, n){
	if (arrayp(arr))
		return arr[n];
}

function length(arr){
	if (arrayp(arr))
		return arr.length;
	if (hashp(arr))
		return len(env.keys(arr));
}

function cons(a, as){
	return [a].concat(as);
}

function car(x){
	if (x.length) return x[0];
}

function cdr(x){
	if (x.length > 1) return x.slice(1);
}

function append(a, as){
	return a.concat(as);
}

function mapf(fn, arr){
	return arr.map(fn);
}

function reducef(fn, arr, init){
	return arr.reduce(function(a, b){
		return fn(a, b);			
	},init);
}
